/*
 * Copyright (C) 2009 Collabora Ltd <http://www.collabora.co.uk>
 * Copyright (C) 2009 Abner Silva <abner.silva@kdemail.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

// Own
#include "accountwidget.h"
#include "accountimage.h"

// Plasma
#include <Plasma/Theme>
#include <Plasma/Label>
#include <Plasma/Frame>

// Kde
//#include <KColorScheme>
#include <KDebug>
#include <KIconLoader>

//Qt
#include <QGraphicsGridLayout>
#include <QLabel>

using namespace Plasma;

AccountWidget::AccountWidget(QGraphicsWidget *parent)
    : Frame(parent),
      m_id(),
      m_image(0),
      m_layout(0),
      m_accountNameLabel(0),
      m_accountAliasLabel(0),
      m_presenceWidget(0)
{
    init();
    setMinimumHeight(75);
    setMinimumWidth(230);
    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
}

AccountWidget::~AccountWidget()
{
}

void AccountWidget::init()
{
    kDebug() << "AccountWidget::init: initializing";

    int m = KIconLoader::SizeMedium;

    m_layout = new QGraphicsGridLayout(this);
    m_layout->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    m_layout->setColumnFixedWidth(0, int(m*1.2));
    m_layout->setHorizontalSpacing(4);

    m_image = new AccountImage(this);
    m_image->setMinimumHeight(m);
    m_image->setMinimumWidth(m);
    m_layout->addItem(m_image, 0, 0, 2, 1, Qt::AlignTop);

    m_accountNameLabel = new Plasma::Label(this);
    m_accountNameLabel->nativeWidget()->setWordWrap(false);
    m_accountNameLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    m_accountNameLabel->setMinimumWidth(m*2);
    m_accountNameLabel->setText("foo@bar.com");
    m_layout->addItem(m_accountNameLabel, 0, 1, 1, 2, Qt::AlignTop);

    m_accountAliasLabel = new Plasma::Label(this);
    m_accountAliasLabel->nativeWidget()->setWordWrap(false);
    m_accountAliasLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    m_accountAliasLabel->setMinimumWidth(m*2);
    m_accountAliasLabel->setText("Mr. Presence Plasmoid");
    m_layout->addItem(m_accountAliasLabel, 1, 1, 1, 2, Qt::AlignCenter | Qt::AlignTop);

    m_presenceWidget = new PresenceWidget(this);
    connect (m_presenceWidget,
            SIGNAL(presenceStatusChanged(PresenceWidget::PresenceStatus,
                                         const QString&)),
            this, SLOT(onPresenceStatusChanged(PresenceWidget::PresenceStatus,
                                               const QString&)));
    m_layout->addItem(m_presenceWidget, 2, 0, 1, 3,
            Qt::AlignBottom | Qt::AlignCenter);

    setLayout(m_layout);
}

void AccountWidget::setId(const QString &id)
{
    m_id = id;
}

QString AccountWidget::id() const
{
    return m_id;
}

void AccountWidget::setName(const QString &name)
{
    m_accountNameLabel->setText(name);
}

QString AccountWidget::name() const
{
    return m_accountNameLabel->text();
}

void AccountWidget::setAlias(const QString &alias)
{
    m_accountAliasLabel->setText(alias);
}

QString AccountWidget::alias() const
{
    return m_accountAliasLabel->text();
}

void AccountWidget::setImage(const QPixmap &image)
{
    m_image->setPixmap(image);
}

void AccountWidget::setPresence(const QString &presence, const QString &msg)
{
    PresenceWidget::PresenceStatus status = PresenceWidget::Offline;

    if (presence.contains("available"))
        status = PresenceWidget::Online;
    else if (presence.contains("away"))
        status = PresenceWidget::Away;
    else if (presence.contains("busy"))
        status = PresenceWidget::Busy;
    else if (presence.contains("invisible"))
        status = PresenceWidget::Invisible;
    else if (presence.contains("offline"))
        status = PresenceWidget::Offline;

    m_presenceWidget->setPresenceStatusWithMsg(status, msg);
}

QString AccountWidget::presenceStatus() const
{
    return presenceStatusToString(m_presenceWidget->presenceStatus());
}

QString AccountWidget::presenceMessage() const
{
    return m_presenceWidget->presenceMessage();
}

void AccountWidget::onPresenceStatusChanged(PresenceWidget::PresenceStatus status,
        const QString &text)
{
    QString ret = presenceStatusToString(status);

    emit presenceChanged(ret, text);
}

QString AccountWidget::presenceStatusToString(PresenceWidget::PresenceStatus status) const
{
    QString ret;

    switch(status) {
        case PresenceWidget::Online:
            ret = "available";
            break;
        case PresenceWidget::Away:
            ret = "away";
            break;
        case PresenceWidget::Busy:
            ret = "busy";
            break;
        case PresenceWidget::Invisible:
            ret = "invisible";
            break;
        case PresenceWidget::Offline:
            ret = "offline";
            break;
        default:
            break;
    }

    return ret;
}
#include "accountwidget.moc"
