/*
 * Copyright (C) 2009 Collabora Ltd <http://www.collabora.co.uk>
 * Copyright (C) 2009 Abner Silva <abner.silva@kdemail.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

// Own
#include "presencewidget.h"

// Qt
#include <QGraphicsLinearLayout>
#include <QLabel>
#include <QTimer>

// Plasma
#include <Plasma/IconWidget>
#include <Plasma/LineEdit>
#include <Plasma/Label>
#include <Plasma/BusyWidget>

// Kde
#include <KIconLoader>
#include <KLineEdit>

const QString DEFAULT_MSG = i18n("Set presence message here");

using namespace Plasma;

PresenceWidget::PresenceWidget(QGraphicsWidget *parent)
    : QGraphicsWidget(parent),
      m_layout(0),
      m_iconsLayout(0),
      m_iconsWidget(0),
      m_currentStatusIcon(0),
      m_currentStatusLayout(0),
      m_currentStatusWidget(0),
      m_presenceMessage(0),
      m_presenceMessageLabel(0),
      m_presenceMsg(""),
      m_waitLayout(0),
      m_waitWidget(0),
      m_waitTimer(0)
{
     init();
}

PresenceWidget::~PresenceWidget()
{
}

void PresenceWidget::init()
{
    // Create the main layout
    m_layout = new QGraphicsLinearLayout(Qt::Vertical, this);
    m_layout->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_layout->setSpacing(0);

    //Create the icons widget
    m_iconsWidget = new QGraphicsWidget(this);
    m_iconsWidget->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

    // Create the icons layout
    m_iconsLayout = new QGraphicsLinearLayout(m_iconsWidget);
    m_iconsLayout->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_iconsLayout->setContentsMargins(0, 0, 0, 0);

    int s = KIconLoader::SizeSmall; // The size for the action icons

    IconWidget *icon = new IconWidget(this);
    icon->setIcon("user-online");
    icon->setToolTip(i18n("Available"));
    icon->setMinimumHeight(s);
    icon->setMaximumHeight(s);
    icon->setMinimumWidth(s);
    icon->setMaximumWidth(s);

    connect(icon, SIGNAL(clicked()),
            this, SLOT(onNewStatusClicked()));

    m_status[PresenceWidget::Online] = icon;
    m_iconsLayout->addItem(icon);

    icon = new IconWidget(this);
    icon->setIcon("user-away");
    icon->setToolTip(i18n("Away"));
    icon->setMinimumHeight(s);
    icon->setMaximumHeight(s);
    icon->setMinimumWidth(s);
    icon->setMaximumWidth(s);

    connect(icon, SIGNAL(clicked()),
            this, SLOT(onNewStatusClicked()));

    m_status[PresenceWidget::Away] = icon;
    m_iconsLayout->addItem(icon);

    icon = new IconWidget(this);
    icon->setIcon("user-busy");
    icon->setToolTip(i18n("Busy"));
    icon->setMinimumHeight(s);
    icon->setMaximumHeight(s);
    icon->setMinimumWidth(s);
    icon->setMaximumWidth(s);

    connect(icon, SIGNAL(clicked()),
            this, SLOT(onNewStatusClicked()));

    m_status[PresenceWidget::Busy] = icon;
    m_iconsLayout->addItem(icon);

    icon = new IconWidget(this);
    icon->setIcon("user-invisible");
    icon->setToolTip(i18n("Invisible"));
    icon->setMinimumHeight(s);
    icon->setMaximumHeight(s);
    icon->setMinimumWidth(s);
    icon->setMaximumWidth(s);

    connect(icon, SIGNAL(clicked()),
            this, SLOT(onNewStatusClicked()));

    m_status[PresenceWidget::Invisible] = icon;
    m_iconsLayout->addItem(icon);

    icon = new IconWidget(this);
    icon->setIcon("user-offline");
    icon->setToolTip(i18n("Offline"));
    icon->setMinimumHeight(s);
    icon->setMaximumHeight(s);
    icon->setMinimumWidth(s);
    icon->setMaximumWidth(s);

    connect(icon, SIGNAL(clicked()),
            this, SLOT(onNewStatusClicked()));

    m_status[PresenceWidget::Offline] = icon;
    m_iconsLayout->addItem(icon);
    m_iconsWidget->setLayout(m_iconsLayout);
    m_iconsWidget->setVisible(false);

    // Create the current status widget
    m_currentStatusWidget = new QGraphicsWidget(this);
    m_currentStatusWidget->setSizePolicy(QSizePolicy::Expanding,
            QSizePolicy::Expanding);

    // Create the current status layout
    m_currentStatusLayout = new QGraphicsLinearLayout(m_currentStatusWidget);
    m_currentStatusLayout->setSizePolicy(QSizePolicy::Expanding,
            QSizePolicy::Expanding);

    m_currentStatusIcon = new IconWidget(this);
    m_currentStatusIcon->setToolTip(i18n("Click to change the status"));
    m_currentStatusIcon->setOrientation(Qt::Horizontal);
    m_currentStatusIcon->setTextBackgroundColor(QColor());
    m_currentStatusIcon->setMinimumHeight(s);
    m_currentStatusIcon->setMaximumHeight(s);
    m_currentStatusIcon->setMinimumWidth(s);
    m_currentStatusIcon->setMaximumWidth(85);

    m_currentStatusIcon->setVisible(false);

    connect(m_currentStatusIcon, SIGNAL(clicked()),
            this, SLOT(onPresenceStatusSwitch()));

    // Create the wait widget
    m_waitWidget = new QGraphicsWidget(this);
    m_waitWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    // Create the wait layout
    m_waitLayout = new QGraphicsLinearLayout(m_waitWidget);
    m_waitLayout->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    BusyWidget *busy = new BusyWidget(this);
    busy->setMinimumHeight(s);
    busy->setMaximumHeight(s);
    busy->setMinimumWidth(s);
    busy->setMaximumWidth(s);

    m_waitLayout->addItem(busy);

    // Create the wait msg label
    Label *waitMessageLabel = new Label(this);
    waitMessageLabel->nativeWidget()->setWordWrap(false);
    waitMessageLabel->nativeWidget()->setAlignment(Qt::AlignTop);
    waitMessageLabel->nativeWidget()->setSizePolicy(
            QSizePolicy::Fixed, QSizePolicy::Fixed);
    waitMessageLabel->setText(i18n("<i>Updating presence</i>"));

    m_waitLayout->addItem(waitMessageLabel);
    m_waitLayout->setContentsMargins(0, 0, 0, 0);
    m_waitWidget->setLayout(m_waitLayout);

    m_waitWidget->setMinimumHeight(s);
    m_waitWidget->setMaximumHeight(s);

    /* FIXME - Workaround =)
     * Sometimes when we try to set up an invalid presence status (like some
     * xmpp server that doesn't supports invisible presence) the data engine
     * will not emit any presence changed signal and the applet will wait
     * forever for that signal. For that reason this timer is needed to avoid
     * this blocking situation. So when it timeouts the applet will return to
     * the last presence status.*/
    m_waitTimer = new QTimer(this);
    m_waitTimer->setInterval(30000);
    connect(m_waitTimer, SIGNAL(timeout()),
            this, SLOT(onPresenceStatusSwitch()));

    m_currentStatusLayout->addItem(m_waitWidget);
    m_currentStatusWidget->setLayout(m_currentStatusLayout);

    m_layout->addItem(m_currentStatusWidget);
    m_layout->setAlignment(m_currentStatusWidget, Qt::AlignCenter);

    // Create the presence msg label
    m_presenceMessageLabel = new Label(this);
    m_presenceMessageLabel->nativeWidget()->setWordWrap(false);
    m_presenceMessageLabel->nativeWidget()->setContentsMargins(1, 1, 1, 1);
    m_presenceMessageLabel->nativeWidget()->setAlignment(Qt::AlignTop |
            Qt::AlignCenter);
    m_presenceMessageLabel->nativeWidget()->setSizePolicy(
            QSizePolicy::Expanding, QSizePolicy::Fixed);
    setPresenceMessage("");

    // Create the presence msg lineedit
    m_presenceMessage = new LineEdit(this);
    m_presenceMessage->setVisible(false);
    m_presenceMessage->nativeWidget()->setText(
            m_presenceMessageLabel->nativeWidget()->text());
    m_presenceMessage->nativeWidget()->setFrame(false);
    m_presenceMessage->nativeWidget()->setSizePolicy(QSizePolicy::Expanding,
            QSizePolicy::Fixed);
    connect(m_presenceMessage->nativeWidget(), SIGNAL(returnPressed()),
            this, SLOT(onPresenceMsgReturn()));

    connect(m_presenceMessageLabel->nativeWidget(),
            SIGNAL(linkActivated(const QString&)),
            this, SLOT(onPresenceMsgSwitch()));

    connect(m_presenceMessage->nativeWidget(),
            SIGNAL(editingFinished()),
            this, SLOT(onPresenceMsgSwitch()));

    m_layout->addItem(m_presenceMessageLabel);
    m_layout->setAlignment(m_presenceMessageLabel, Qt::AlignTop |
            Qt::AlignCenter);

    setLayout(m_layout);
}

void PresenceWidget::onNewStatusClicked()
{
    PresenceStatus status = m_status.key(
            static_cast<IconWidget *>(sender()));

    onPresenceStatusSwitch();

    if (m_currentStatus == status)
        onPresenceStatusSwitch();
    else
        emit presenceStatusChanged(status, m_presenceMsg);
}

void PresenceWidget::onPresenceStatusSwitch()
{
    if (m_currentStatusIcon->isVisible()) {
        m_currentStatusIcon->hide();
        m_currentStatusLayout->removeItem(m_currentStatusIcon);

        m_currentStatusLayout->addItem(m_iconsWidget);
        m_iconsWidget->show();
    }
    else if (m_waitWidget->isVisible()) {
        m_waitTimer->stop();
        m_waitWidget->hide();
        m_currentStatusLayout->removeItem(m_waitWidget);

        m_currentStatusLayout->addItem(m_currentStatusIcon);
        m_currentStatusIcon->show();
    }
    else if (m_iconsWidget->isVisible()) {
        m_iconsWidget->hide();
        m_currentStatusLayout->removeItem(m_iconsWidget);

        m_currentStatusLayout->addItem(m_waitWidget);
        m_waitWidget->show();
        m_waitTimer->start();
    }
}

void PresenceWidget::onPresenceMsgReturn()
{
    if (m_presenceMsg != m_presenceMessage->nativeWidget()->text()) {
        setPresenceMessage(m_presenceMessage->nativeWidget()->text());
        emit presenceStatusChanged(m_currentStatus, m_presenceMsg);
    }
}

void PresenceWidget::onPresenceMsgSwitch()
{
    if (m_presenceMessageLabel->isVisible()) {
        m_presenceMessageLabel->hide();
        m_layout->removeItem(m_presenceMessageLabel);

        m_layout->addItem(m_presenceMessage);
        m_layout->setAlignment(m_presenceMessage, Qt::AlignTop |
                Qt::AlignCenter);
        m_presenceMessage->nativeWidget()->setText(m_presenceMsg);
        m_presenceMessage->show();
        m_presenceMessage->nativeWidget()->setFocus();
    }
    else {
        m_presenceMessage->hide();
        m_layout->removeItem(m_presenceMessage);

        m_layout->addItem(m_presenceMessageLabel);
        m_layout->setAlignment(m_presenceMessageLabel, Qt::AlignTop |
                Qt::AlignCenter);

        m_presenceMessageLabel->show();
    }
}

void PresenceWidget::setPresenceMessage(const QString &msg)
{
    QString realMsg = msg;

    if (realMsg.isEmpty())
        realMsg = DEFAULT_MSG;

    m_presenceMessageLabel->nativeWidget()->setText(
            QString("<a href=a>%1</a>").arg(realMsg));

    m_presenceMsg = msg;
}

const QString PresenceWidget::presenceMessage() const
{
    return m_presenceMsg;
}

void PresenceWidget::setPresenceStatus(PresenceStatus status)
{
    if (m_currentStatus != status) {
        IconWidget *icon = m_status[status];

        m_currentStatusIcon->setIcon(icon->icon());
        m_currentStatusIcon->setText(icon->toolTip());

        m_currentStatus = status;
    }

    if (m_iconsWidget->isVisible() || m_waitWidget->isVisible())
        onPresenceStatusSwitch();
}

PresenceWidget::PresenceStatus PresenceWidget::presenceStatus() const
{
    return m_currentStatus;
}

void PresenceWidget::setPresenceStatusWithMsg(PresenceStatus status,
        const QString &msg)
{
    setPresenceStatus(status);
    setPresenceMessage(msg);
}
